package com.bocloud.sms.service;

import com.bocloud.sms.entity.Admission;
import com.bocloud.sms.interfaces.AdmissionService;
import com.bocloud.sms.model.AdmissionModel;
import com.bocloud.sms.repository.AdmissionRepository;
import com.megatron.common.exception.InternalServerException;
import com.megatron.common.model.*;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * ip访问控制业务实现
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/12
 */
@Service("admissionService")
@Slf4j
public class AdmissionServiceImpl implements AdmissionService {

    private final AdmissionRepository admissionRepository;
    private final LockFactory lockFactory;

    public AdmissionServiceImpl(AdmissionRepository admissionRepository, LockFactory lockFactory) {
        this.admissionRepository = admissionRepository;
        this.lockFactory = lockFactory;
    }

    /**
     * 查询IP访问限制列表
     *
     * @param pager
     * @return
     */
    @Override
    public GeneralResult<GridBean<Admission>> list(Pager pager) {
        List<Admission> list;
        int total;
        GridBean<Admission> gridBean;
        try {
            total = admissionRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<Admission> beans = this.admissionRepository.list(pager.getParams());
                gridBean = new GridBean<>(1, 1, total, beans);
            } else {
                list = this.admissionRepository.list(pager);
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("list ip access failure:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 查询IP访问限制详情
     *
     * @param id
     * @return
     */
    @Override
    public GeneralResult<Admission> detail(Long id) {
        try {
            Admission admission = this.admissionRepository.query(id);
            return new GeneralResult<>(true, admission, "查询成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 创建IP访问限制
     *
     * @param admissionModel
     * @param requestContext
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result create(AdmissionModel admissionModel, RequestContext requestContext) throws InternalServerException {
        try {
            Admission admission = new Admission();
            BeanUtils.copyProperties(admissionModel, admission);
            Admission exist = admissionRepository.getByIp(admission.getIp());
            Assert.isNull(exist, "该IP地址已存在，添加失败");
            admission.setCreatorId(requestContext.getTarget());
            admission.setGmtCreate(new Date());
            admissionRepository.save(admission);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            log.error("Get error message：", e);
            throw new InternalServerException("创建IP访问限制失败！" + e.getMessage(), e, e.getMessage());
        }
    }

    /**
     * 删除IP访问限制
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result remove(Long id) throws InternalServerException {
        String path = Admission.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Admission access = this.admissionRepository.query(id);
            Assert.notNull(access, "IP访问限制数据不存在");
            this.admissionRepository.remove(id);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove ip access error:", e);
            throw new InternalServerException("删除IP访问限制失败！", e, e.getMessage());
        }
    }

    /**
     * 批量删除IP访问限制
     *
     * @param ids
     * @return
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class)
    public Result batchRemove(List<Long> ids) throws InternalServerException {
        try {
            Assert.notEmpty(ids, "没有要删除的的IP");
            for (Long id : ids) {
                this.remove(id);
            }
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove ip access error:", e);
            throw new InternalServerException("批量删除IP访问限制失败！", e, e.getMessage());
        }
    }
}
