package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.sms.entity.Permission;
import com.bocloud.sms.entity.Plugin;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.ManagerService;
import com.bocloud.sms.interfaces.PluginService;
import com.bocloud.sms.model.AppModule;
import com.bocloud.sms.model.PermissionBean;
import com.bocloud.sms.model.single.RoleResponseModel;
import com.bocloud.sms.repository.PermissionRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.utils.ServiceWatcher;
import com.bocloud.sms.utils.PermitComparator;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/2/4 周二 12:50
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class ManagerServiceImpl implements ManagerService {

    private final PermissionRepository permissionRepository;
    private final PluginService pluginService;
    private final ServiceWatcher serviceWatcher;

    /**
     * 菜单权限是否使用插件校验
     */
    @Value("${plugin.permission.enable:true}")
    private boolean validatePlugin;
    /**
     * 是否使用权限系统的菜单
     */
    @Value("${single.role.enable:false}")
    private boolean singleRoleEnable;

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private TrxServiceImpl trxServiceImpl;

    private String getRedisKey(long userId) {
        return "single_user_roles_"+ userId;
    }

    /**
     * 查询管理员权限树形列表
     *
     * @param context
     * @return
     */
    @Override
    public GeneralResult<List<PermissionBean>> listPermissions(RequestContext context, String trxToken) {
        // 处里天融信用户登录
        User user = userRepository.query(context.getTarget());
        List<Long> qxAppModuleIds = new ArrayList<>();
        if (null != user.getUserId()) {
            try {
                // 天融信用户获取权限系统菜单权限
                log.info("开始获取天融信登录用户所拥有权限系统列表");
                List<AppModule> appModules = trxServiceImpl.queryQxCategoryList(trxToken, user.getUserId());
                if (CollectionUtils.isEmpty(appModules)) {
                    log.error("获取天融信登录用户所拥有权限系统列表失败");
                    return new GeneralResult<>(false, "获取权限系统菜单信息失败");
                }
                for (AppModule appModule : appModules) {
                    long id = Long.parseLong(appModule.getId());
                    qxAppModuleIds.add(id);
                }
            } catch (Exception e) {
                log.error("获取天融信登录用户所拥有权限系统列表失败:", e);
            }
        }

        // 查询所拥有权限(菜单，按钮，api)
        List<Permission> permissions = permissionRepository.getPermissions(context.getTarget());

        // 如果为天融信登录用户，那么在云管过滤出天融信登录用户所拥有权限系统权限
        if (!CollectionUtils.isEmpty(qxAppModuleIds)) {
            permissions = permissions.stream()
                    .filter(permission -> qxAppModuleIds.contains(permission.getId()))
                    .collect(Collectors.toList());
        }

        // 查询所有权限
        Map<Long, Permission> allPermissions = permissionRepository.list(false).stream()
                .collect(Collectors.toMap(Permission::getId, Function.identity()));
        List<Permission> parents = new ArrayList<>();
        for (Permission permission : permissions) {
            if ("menu".equals(permission.getCategory()) || "view".equals(permission.getCategory())) {
                //获取父权限
                this.getParents(allPermissions, parents, permission);
            }
        }
        if (!ListTool.isEmpty(parents)) {
            permissions.addAll(parents);
        }
        //权限去重
        Map<Long, Permission> permissionMap = new HashMap<>(permissions.size(), 0.75F);
        for (Permission permission : permissions) {
            permissionMap.put(permission.getId(), permission);
        }
        permissions = new ArrayList<>(permissionMap.values());
        Map<Long, PermissionBean> permission = new HashMap<>(permissions.size(), 0.75F);
        //重新构建返回权限数据实体，并去重
        for (Permission permit : permissions) {
            if ("menu".equals(permit.getCategory()) || "view".equals(permit.getCategory())) {
                PermissionBean permissionBean = new PermissionBean(permit.getId(), permit.getName(), permit.getPath(),
                        permit.getIcon(), permit.getParentId(), permit.getPriority(), permit.getRouter(),
                        permit.getComponent(), permit.getRedirect(), permit.getMeta(), permit.getParams(),
                        permit.getCategory(), permit.getPluginsCode(), permit.getPluginsStatus(), permit.getButtonCode());
                permission.put(permissionBean.getId(), permissionBean);
            }
        }
        // 菜单权限
        List<PermissionBean> permissionBeans = new ArrayList<>(permission.values());
        Set<PermissionBean> resultPermission = new HashSet<>();
        // 当前运行中的插件
        List<Plugin> plugins = this.pluginService.runningPlugins(BoCloudService.CMP.getName(), BoCloudService.COS.getName(),
                BoCloudService.COP.getName());
        if (plugins == null || !validatePlugin) {
            // 对权限菜单进行递归排序
            List<PermissionBean> menus = getPermissions(0L, permissionBeans);
            return new GeneralResult<>(true, menus, "获取菜单权限信息成功");
        } else {
            Optional.of(permissionBeans).orElse(new ArrayList<>()).forEach(permissionBean -> {
                if (StringUtils.isBlank(permissionBean.getPluginsCode())) {
                    resultPermission.add(permissionBean);
                    return;
                }
                Optional.of(plugins).orElse(new ArrayList<>()).forEach(plugin -> {
                    // 获取到插件编码
                    String beanPluginsCode = permissionBean.getPluginsCode();
                    // 如果菜单绑定插件为空或者为开启状态，则进行添加操作
                    boolean pluginsStatus = permissionBean.getPluginsStatus() != null && permissionBean.getPluginsStatus();
                    if (pluginsStatus && beanPluginsCode.equals(plugin.getCode())) {
                        resultPermission.add(permissionBean);
                    }
                });
            });
            // 对权限菜单进行递归排序
            List<PermissionBean> menus = getPermissions(0L, new ArrayList<>(resultPermission));
            // 过滤权限系统不存在的菜单
            List<PermissionBean> permissionBeans1 = comparePermission(menus, context.getTarget());
            return new GeneralResult<>(true, permissionBeans1, "获取菜单权限信息成功");
        }
    }

    private List<PermissionBean> comparePermission(List<PermissionBean> menus, Long userId) {
        User user = userRepository.query(userId);
        Asserts.notNull(user, "用户数据不存在");
        if (!singleRoleEnable || StringUtils.isEmpty(user.getUserId())) {
            return menus;
        }
        String redisValue = redisTemplate.opsForValue().get(getRedisKey(userId));
        List<RoleResponseModel.Role> roles = JSONArray.parseArray(redisValue, RoleResponseModel.Role.class);

        return compare(menus, roles, 1);
    }

    private List<PermissionBean> compare(List<PermissionBean> menus, List<RoleResponseModel.Role> roles, int level) {
        if (level > 3) {
            return menus;
        }
        List<PermissionBean> returnBeans = new ArrayList<>();

        if (ListTool.isEmpty(menus)) {
            return menus;
        }

        for (PermissionBean permissionBean : menus) {
            Optional<RoleResponseModel.Role> first =
                    Optional.ofNullable(roles).orElse(new ArrayList<>()).stream()
                            .filter(role ->
                                    role.getName().equals(permissionBean.getName()))
                            .findFirst();
            if (first.isPresent()) {
                level++;
                List<PermissionBean> childBean = compare(permissionBean.getChildren(), first.get().getChildren(), level);
                permissionBean.setChildren(childBean);
                returnBeans.add(permissionBean);
            }
        }
        return returnBeans;
    }

    /**
     * 递归查询父菜单
     *
     * @param allPermissionMap 所有权限集合
     * @param parents          返回目标父权限集合
     * @param permission       需查询的权限
     * @return
     */
    private List<Permission> getParents(Map<Long, Permission> allPermissionMap, List<Permission> parents,
                                        Permission permission) {
        Long parentId = permission.getParentId();
        //当前权限没有父权限id(递归到顶层) ，则添加到父权限集合，到达递归出口，返回父权限集合
        if (null == parentId || 0L == parentId) {
            parents.add(permission);
            return parents;
        } else {
            //当前权限有父权限id，则获取当前权限的父权限，添加到返回集合，并再次递归
            try {
                Permission parent = allPermissionMap.get(parentId);
                if (null != parent) {
                    parents.add(parent);
                }
                return this.getParents(allPermissionMap, parents, parent);
            } catch (Exception e) {
                log.error("get Permission failure:", e);
                return null;
            }
        }
    }


    /**
     * 对管理员权限进行递归排序
     *
     * @param pid             父权限id
     * @param permissionBeans 权限集合
     * @return
     */
    private List<PermissionBean> getPermissions(Long pid, List<PermissionBean> permissionBeans) {
        List<PermissionBean> newPermissions = Lists.newArrayList();
        for (PermissionBean permissionBean : permissionBeans) {
            //目标权限id等于父权限id，则进入递归获取其子权限集合
            if (null != permissionBean.getParentId() && permissionBean.getParentId().equals(pid)) {
                List<PermissionBean> children = getPermissions(permissionBean.getId(), permissionBeans);
                //子权限不为空时则对权限进行排序后set到父权限下
                if (!ListTool.isEmpty(children)) {
                    children.sort(new PermitComparator());
                    permissionBean.setChildren(children);
                } else {
                    //子权限为空
                    permissionBean.setChildren(null);
                }
                // 根据当前服务控制显示的菜单
                if (StringUtils.isNotBlank(permissionBean.getMeta())) {
                    JSON.parseArray(permissionBean.getMeta(), Map.class).forEach(metaItem -> {
                        if ("service".equals(metaItem.get("key"))) {
                            String service = String.valueOf(metaItem.get("value"));
                            if (StringUtils.isBlank(service)) {
                                return;
                            }
                            if (!serviceWatcher.getServices().contains(service)) {
                                permissionBean.setCategory("view");
                            }
                        }
                    });
                }
                //添加到返回目标集合
                newPermissions.add(permissionBean);
            }
        }
        newPermissions.sort(new PermitComparator());
        return newPermissions;
    }

}
