package com.bocloud.sms.service;

import com.bocloud.sms.entity.SystemNotice;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.SystemNoticeService;
import com.bocloud.sms.model.NotifyMessage;
import com.bocloud.sms.repository.SystemNoticeRepository;
import com.bocloud.sms.repository.UserRepository;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service("SystemNoticeService")
@Slf4j
public class SystemNoticeServiceImpl implements SystemNoticeService {

    private final SystemNoticeRepository systemNoticeRepository;
    private final LockFactory lockFactory;
    private final AmqpTemplate amqpTemplate;
    private final UserRepository userRepository;

    public SystemNoticeServiceImpl(SystemNoticeRepository systemNoticeRepository, LockFactory lockFactory, AmqpTemplate amqpTemplate, UserRepository userRepository) {
        this.systemNoticeRepository = systemNoticeRepository;
        this.lockFactory = lockFactory;
        this.amqpTemplate = amqpTemplate;
        this.userRepository = userRepository;
    }

    @Override
    public Result create(SystemNotice entity, RequestContext user) {
        try {
            log.info("[创建]系统公告:{}", entity);
            Assert.notNull(entity, "参数格式错误");
            // 创建数据
            entity.setCreatorId(user.getTarget());
            entity.setMenderId(user.getTarget());
            entity.setGmtCreate(new Date());
            entity.setGmtModify(new Date());
            entity.setStatus(StatusEnum.NOTSEND.name());
            entity.setDeleted(false);
            if (entity.getSendType().equals(SendTypeEnum.NOW.name())) {
                entity.setSendTime(new Date());
            }
            systemNoticeRepository.save(entity);
            //执行发送
            if (entity.getSendType().equals(SendTypeEnum.NOW.name())) {
                sendMsg(entity.getId());
            }
        } catch (Exception e) {
            log.error("创建系统公告 error:", e);
            return new Result(false, "创建系统公告失败:" + e.getMessage());
        }
        return new Result(true, "添加成功");
    }

    @Override
    public Result remove(Long id, RequestContext user) {
        log.info("[删除]系统公告:{}", id);
        String path = SystemNotice.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SystemNotice entity = systemNoticeRepository.query(id);
            Assert.notNull(entity, "数据不存在");
            if (StatusEnum.SEND.name().equals(entity.getStatus())) {
                return new GeneralResult<>(false, "已发送的公告不允许删除");
            }
            systemNoticeRepository.remove(SystemNotice.class, id);
        } catch (Exception e) {
            log.error("Remove SystemNotice error:", e);
            return new Result(false, "操作失败");
        }
        return new Result(true, "操作成功");
    }

    @Override
    public Result modify(SystemNotice params, RequestContext user) {
        log.info("[修改]系统公告:{}", params);
        Long id = params.getId();
        String path = SystemNotice.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            SystemNotice entity = systemNoticeRepository.query(id);
            Assert.notNull(entity, "数据不存在");
            if (StatusEnum.SEND.name().equals(entity.getStatus())) {
                return new GeneralResult<>(false, "已发送的公告不允许编辑");
            }
            BeanUtils.copyProperties(entity, params);
            entity.setMenderId(user.getTarget());
            entity.setGmtModify(new Date());
            entity.setDeleted(false);
            if (entity.getSendType().equals(SendTypeEnum.NOW.name())) {
                entity.setSendTime(new Date());
            }
            systemNoticeRepository.update(entity);
            //执行发送
            if (entity.getSendType().equals(SendTypeEnum.NOW.name()) && entity.getStatus().equals(StatusEnum.NOTSEND.name())) {
                sendMsg(entity.getId());
            }
        } catch (Exception e) {
            log.error("modify SystemNotice error:", e);
            return new Result(false, "操作失败");
        }
        return new Result(true, "操作成功");
    }

    @Override
    public GeneralResult<GridBean<SystemNotice>> list(Pager pager, RequestContext context) {
        try {
            log.info("[查询]系统公告列表:{}", pager);
            List<SystemNotice> list;
            int total;
            GridBean<SystemNotice> gridBean;
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            Boolean simple = Optional.ofNullable(pager.getSimple()).orElse(false);
            int page = Optional.ofNullable(pager.getPage()).orElse(1);
            int rows = Optional.ofNullable(pager.getRows()).orElse(10);
            total = systemNoticeRepository.count(params);
            list = systemNoticeRepository.list(page, rows, params, sorter);
            for (SystemNotice notice : list) {
                User user = userRepository.query(notice.getCreatorId());
                if (null != user) {
                    notice.setCreatorName(user.getName());
                }
            }
            gridBean = new GridBean<>(page, rows, total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error(" list SystemNotice error:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<SystemNotice> detail(Long id) {
        try {
            log.info("[查询]系统公告详情:{}", id);
            Assert.notNull(id, "参数格式错误");
            SystemNotice entity = systemNoticeRepository.query(id);
            Assert.notNull(entity, "数据不存在");
            User user = userRepository.query(entity.getCreatorId());
            if (null != user) {
                entity.setCreatorName(user.getName());
            }
            return new GeneralResult<>(true, entity, "查询成功");
        } catch (Exception e) {
            log.error("Get SystemNotice detail error message：", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 发送系统公告
     */
    @Override
    public GeneralResult<Void> sendMsg(Long id) {
        try {
            log.info("[执行]系统公告发送：{}", id);
            Assert.notNull(id, "参数格式错误");
            SystemNotice entity = systemNoticeRepository.query(id);
            Assert.notNull(entity, "数据不存在");
            if (StatusEnum.SEND.name().equals(entity.getStatus())) {
                return new GeneralResult<>(false, "请勿重复操作");
            }
            List<User> users = new ArrayList<>();
            // 发送范围
            List<String> sendGroups = Arrays.asList(entity.getSendGroups().split(","));
            // 勾选全部  或者  同时勾选 管理端和租户端
            if (sendGroups.contains(SendGroupEnum.ALL.name()) || (sendGroups.contains(SendGroupEnum.MANAGE.name()) && sendGroups.contains(SendGroupEnum.TENANT.name()))) {
                // 查询所有用户
                users = userRepository.list(1, Integer.MAX_VALUE, new ArrayList<>(), new HashMap<String, String>(8), null);
            } else if (sendGroups.contains(SendGroupEnum.MANAGE.name())) {
                // 查询管理端用户
                List<Param> params = Lists.newArrayList();
                params.add(new Param(MapTools.simpleMap("isManager", 1), Sign.EQ));
                users = userRepository.list(1, Integer.MAX_VALUE, params, new HashMap<String, String>(8), null);
            } else if (sendGroups.contains(SendGroupEnum.TENANT.name())) {
                // 查询租户端用户
                List<Param> params = Lists.newArrayList();
                params.add(new Param(MapTools.simpleMap("isManager", 0), Sign.EQ));
                users = userRepository.list(1, Integer.MAX_VALUE, params, new HashMap<String, String>(8), null);
            }
            if (sendGroups.contains(SendGroupEnum.INPUT.name())) {
                // 添加指定的用户
                String regex = ";|；";
                Map<String, List<String>> maps = Maps.newHashMap();
                if (StringUtils.hasText(entity.getAccounts())) {
                    maps.put("account", Arrays.asList(entity.getAccounts().split(regex)));
                }
                if (StringUtils.hasText(entity.getEmails())) {
                    maps.put("email", Arrays.asList(entity.getEmails().split(regex)));
                }
                if (StringUtils.hasText(entity.getDings())) {
                    maps.put("dingtalk", Arrays.asList(entity.getDings().split(regex)));
                }
                if (StringUtils.hasText(entity.getWechats())) {
                    maps.put("wechat", Arrays.asList(entity.getWechats().split(regex)));
                }
                if (StringUtils.hasText(entity.getPhones())) {
                    maps.put("phone", Arrays.asList(entity.getPhones().split(regex)));
                }

                for (Map.Entry<String, List<String>> entry : maps.entrySet()) {
                    String k = entry.getKey();
                    List<String> v = entry.getValue();
                    for (String var : v) {
                        if (!StringUtils.hasText(var)) {
                            continue;
                        }
                        List<Param> params = Lists.newArrayList();
                        params.add(new Param(MapTools.simpleMap(k, var), Sign.EQ));
                        List<User> u = userRepository.list(1, 1, params, new HashMap<>(8), null);
                        if (u.size() < 1) {
                            log.info("找不到指定用户：{}", var);
                            continue;
                        }
                        // 校验 users 中是否已经存在 user
                        Map<String, User> userMap = users.stream().collect(Collectors.toMap(User::getAccount, Function.identity()));
                        User user = u.get(0);
                        if (userMap.containsKey(user.getAccount())) {
                            continue;
                        }
                        users.add(user);
                    }
                }

               /* // 指定账号
                if (StringUtils.hasText(entity.getAccounts())) {
                    List<String> inputAccounts = Arrays.asList(entity.getAccounts().split(regex));
                    for (String var : inputAccounts) {
                        List<Param> params = Lists.newArrayList();
                        params.add(new Param(MapTools.simpleMap("account", var), Sign.EQ));
                        List<User> u = userRepository.list(1, 1, params, new HashMap<String, String>(8), null);
                        if (u.size() < 1) {
                            log.info("指定账号不存在：{}", var);
                            continue;
                        }
                        users.add(u.get(0));
                    }
                }
                // 指定邮箱
                if (StringUtils.hasText(entity.getEmails())) {
                    List<String> inputEmails = Arrays.asList(entity.getEmails().split(regex));
                    for (String var : inputEmails) {
                        List<Param> params = Lists.newArrayList();
                        params.add(new Param(MapTools.simpleMap("email", var), Sign.EQ));
                        List<User> u = userRepository.list(1, 1, params, new HashMap<String, String>(8), null);
                        if (u.size() < 1) {
                            log.info("指定账号不存在：{}", var);
                            continue;
                        }
                        users.add(u.get(0));
                    }
                }
                // 指定钉钉
                if (StringUtils.hasText(entity.getDings())) {
                    List<String> inputDings = Arrays.asList(entity.getDings().split(regex));
                    for (String var : inputDings) {
                        List<Param> params = Lists.newArrayList();
                        params.add(new Param(MapTools.simpleMap("dingtalk", var), Sign.EQ));
                        List<User> u = userRepository.list(1, 1, params, new HashMap<String, String>(8), null);
                        if (u.size() < 1) {
                            log.info("指定账号不存在：{}", var);
                            continue;
                        }
                        users.add(u.get(0));
                    }
                }
                // 指定微信
                if (StringUtils.hasText(entity.getWechats())) {
                    List<String> inputWehats = Arrays.asList(entity.getWechats().split(regex));
                    for (String var : inputWehats) {
                        List<Param> params = Lists.newArrayList();
                        params.add(new Param(MapTools.simpleMap("wechat", var), Sign.EQ));
                        List<User> u = userRepository.list(1, 1, params, new HashMap<String, String>(8), null);
                        if (u.size() < 1) {
                            log.info("指定账号不存在：{}", var);
                            continue;
                        }
                        users.add(u.get(0));
                    }
                }
                // 指定手机号码
                if (StringUtils.hasText(entity.getPhones())) {
                    List<String> inputPhones = Arrays.asList(entity.getPhones().split(regex));
                    for (String var : inputPhones) {
                        List<Param> params = Lists.newArrayList();
                        params.add(new Param(MapTools.simpleMap("phone", var), Sign.EQ));
                        List<User> u = userRepository.list(1, 1, params, new HashMap<String, String>(8), null);
                        if (u.size() < 1) {
                            log.info("指定账号不存在：{}", var);
                            continue;
                        }
                        users.add(u.get(0));
                    }
                }
*/
            }
            // 更新发送状态
            entity.setStatus(StatusEnum.SEND.name());
            systemNoticeRepository.update(entity);
            // 发送方式
            // 邮件
            boolean sendEmail = entity.getSendWays().contains(SendwayEnum.EMAIL.name());
            // 站内信
            boolean sendMsg = entity.getSendWays().contains(SendwayEnum.MESSAGE.name());
            // 企业微信
            boolean sendWeChat = entity.getSendWays().contains(SendwayEnum.WECHAT.name());
            // 钉钉
            boolean sendDing = entity.getSendWays().contains(SendwayEnum.DING.name());

            for (User user : users) {
                RequestContext context = new RequestContext(RequestContext.Catalog.User, user.getId(), null, null, null);
                NotifyMessage message = new NotifyMessage(context, entity.getTitle(), entity.getContent(), sendMsg, sendEmail);
                message.setSendWeChat(sendWeChat);
                message.setSendDingTalk(sendDing);
                amqpTemplate.convertAndSend("event.notice.queue", message);
            }
        } catch (Exception e) {
            log.error("Send message error: {}", e.getMessage(), e);
            return new GeneralResult<>(false, "操作失败");
        }
        return new GeneralResult<>(true, "操作成功");
    }

    // 状态
    public enum StatusEnum {
        // 待发送
        NOTSEND,
        //已发送
        SEND
    }

    // 发送时间
    public enum SendTypeEnum {
        //立即
        NOW,
        //定时
        CLOCK
    }

    // 发送方式
    public enum SendwayEnum {
        //站内信
        MESSAGE,
        //邮件
        EMAIL,
        //钉钉
        DING,
        //企业微信
        WECHAT,
        //短信
        SMS
    }

    // 发送范围
    public enum SendGroupEnum {
        //全部
        ALL,
        //管理端
        MANAGE,
        //租户端
        TENANT,
        //指定
        INPUT
    }

}
