package com.bocloud.sms.service.utils;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.builder.ExcelWriterBuilder;
import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.*;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;


/**
 * @author tyl
 * @time 2020/9/8 周二 16:19
 */
@Slf4j
public class ExportUtil {

    /**
     * 导出Excel
     *
     * @param sheetName sheet名称
     * @param title     标题
     * @param values    内容
     * @return
     */
    public static XSSFWorkbook getXssfWorkbook(String sheetName, String[] title, String[][] values) {
        // 第一步，创建一个HSSFWorkbook，对应一个Excel文件
        XSSFWorkbook wb = new XSSFWorkbook();
        // 第二步，在workbook中添加一个sheet,对应Excel文件中的sheet
        XSSFSheet sheet = wb.createSheet(sheetName);
        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
        XSSFRow row = sheet.createRow(0);
        // 第四步，创建单元格，并设置值表头 设置表头居中
        XSSFCellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
        // 声明列对象
        XSSFCell cell;
        // 创建标题
        for (int i = 0; i < title.length; i++) {
            cell = row.createCell(i);
            cell.setCellValue(title[i]);
            cell.setCellStyle(style);
            sheet.setColumnWidth(i, 20 * 256);
        }
        // 创建内容
        for (int i = 0; i < values.length; i++) {
            row = sheet.createRow(i + 1);
            for (int j = 0; j < values[i].length; j++) {
                // 将内容按顺序赋给对应的列对象
                row.createCell(j).setCellValue(values[i][j]);
            }
        }
        return wb;
    }

    public static HSSFWorkbook getHssfWorkbook(String sheetName, String[] title, String[][] values) {
        // 第一步，创建一个HSSFWorkbook，对应一个Excel文件
        HSSFWorkbook wb = new HSSFWorkbook();
        // 第二步，在workbook中添加一个sheet,对应Excel文件中的sheet
        HSSFSheet sheet = wb.createSheet(sheetName);
        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
        HSSFRow row = sheet.createRow(0);
        // 第四步，创建单元格，并设置值表头 设置表头居中
        HSSFCellStyle style = wb.createCellStyle();
        style.setAlignment(HorizontalAlignment.CENTER); // 创建一个居中格式
        // 声明列对象
        HSSFCell cell;
        // 创建标题
        for (int i = 0; i < title.length; i++) {
            cell = row.createCell(i);
            cell.setCellValue(title[i]);
            cell.setCellStyle(style);
            sheet.setColumnWidth(i, 20 * 256);
        }
        // 创建内容
        for (int i = 0; i < values.length; i++) {
            row = sheet.createRow(i + 1);
            for (int j = 0; j < values[i].length; j++) {
                // 将内容按顺序赋给对应的列对象
                row.createCell(j).setCellValue(values[i][j]);
            }
        }
        return wb;
    }

    public static void addValidation(XSSFSheet sheet, List<String> data, Integer colNum) {
        String[] dataList = data.toArray(new String[0]);
        CellRangeAddressList typeList = new CellRangeAddressList(1, 5000, colNum, colNum);
        //加载下拉列表
        XSSFDataValidationHelper typeValidationHelper = new XSSFDataValidationHelper(sheet);
        DataValidationConstraint typeConstraint = typeValidationHelper.createExplicitListConstraint(dataList);
        //数据有效性对象
        DataValidation typeDataValidation = typeValidationHelper.createValidation(typeConstraint, typeList);
        //设置输入错误提示信息
        typeDataValidation.createErrorBox("非法输入", "不允许输入，请选取下拉值！");
        sheet.addValidationData(typeDataValidation);
    }

    public static String getCellStringValue(Cell cell) {
        if (cell == null) {
            return "";
        }
        CellType type = cell.getCellTypeEnum();
        String cellValue;
        switch (type) {
            case BOOLEAN:
                cellValue = String.valueOf(cell.getBooleanCellValue());
                break;
            case NUMERIC:
                cellValue = BigDecimal.valueOf(cell.getNumericCellValue()).toString();
                break;
            case STRING:
                cellValue = cell.getStringCellValue();
                break;
            case FORMULA:
                cellValue = cell.getCellFormula();
                break;
            default:
                cellValue = "";
                break;
        }
        return cellValue;
    }

    /**
     * 导出Excel
     *
     * @param sheetName        sheet名称
     * @param reFileName       文件名称
     * @param data             数据
     * @param clazz            类型
     * @param cellWriteHandler 单元格写处理
     * @return
     */
    public static <T> boolean exportExcel(String reFileName, String sheetName, HttpServletResponse response, List<T> data, Class<T> clazz, CellWriteHandler cellWriteHandler) {
        try {
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setCharacterEncoding("utf-8");
            String fileName = URLEncoder.encode(reFileName, StandardCharsets.UTF_8).replaceAll("\\+", "%20");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
            WriteCellStyle writeCellStyle = new WriteCellStyle();
            writeCellStyle.setHorizontalAlignment(HorizontalAlignment.CENTER);
            WriteFont headWriteFont = new WriteFont();
            headWriteFont.setFontHeightInPoints((short) 11);
            headWriteFont.setBold(false);
            writeCellStyle.setWriteFont(headWriteFont);
            writeCellStyle.setFillForegroundColor(IndexedColors.WHITE.getIndex());
            HorizontalCellStyleStrategy horizontalCellStyleStrategy = new HorizontalCellStyleStrategy(writeCellStyle, writeCellStyle);
            ExcelWriterBuilder write = EasyExcel.write(response.getOutputStream(), clazz).registerWriteHandler(horizontalCellStyleStrategy);
            if (cellWriteHandler != null) {
                write.registerWriteHandler(cellWriteHandler);
            }
            write.useDefaultStyle(false).sheet(sheetName).doWrite(data);
            return true;
        } catch (IOException e) {
            log.error("导出excel异常", e);
            return false;
        }
    }

    /**
     * 导出Excel时设置下拉列表
     *
     * @param map key 表示需要填充下拉列表列的 index，value 表示下拉数据
     * @return
     */
    public static CellWriteHandler getCellWriteHandler(Map<Integer, List<String>> map) {
        return new CellWriteHandler() {
            @Override
            public void beforeCellCreate(WriteSheetHolder writeSheetHolder, WriteTableHolder writeTableHolder, Row row, Head head, Integer columnIndex, Integer relativeRowIndex, Boolean isHead) {
            }

            @Override
            public void afterCellCreate(WriteSheetHolder writeSheetHolder, WriteTableHolder writeTableHolder, Cell cell, Head head, Integer relativeRowIndex, Boolean isHead) {
                Sheet sheet = writeSheetHolder.getSheet();
                // 获取数据校验器
                DataValidationHelper helper = sheet.getDataValidationHelper();
                for (Integer integer : map.keySet()) {
                    // 设置下拉列表的位置
                    CellRangeAddressList cellRangeAddressList = new CellRangeAddressList(1, 100, integer, integer);
                    // 创建约束
                    DataValidationConstraint constraint = helper.createExplicitListConstraint(map.get(integer).toArray(new String[]{}));
                    // 创建验证
                    DataValidation validation = helper.createValidation(constraint, cellRangeAddressList);
                    // 处理Excel兼容性问题
                    if (validation instanceof XSSFDataValidation) {
                        validation.setSuppressDropDownArrow(true);
                        validation.setShowErrorBox(true);
                    } else {
                        validation.setSuppressDropDownArrow(false);
                    }
                    sheet.addValidationData(validation);
                }
            }
        };
    }
}
